# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''Manipulate tag objects for Firmant.
'''


import os

from firmant.parsers import RstParser
from firmant.parsers import RstObject


__all__ = ['TagParser']


class TagParser(RstParser):
    r'''Interpret *.rst for a given tag directory.

    For each reSt document, parse it to a tag and extract its metadata.

        >>> t = TagParser()
        >>> t = t.parse_one('content/tags/rcos.rst')
        >>> t.slug
        u'rcos'
        >>> t.title
        u'Rensselaer Center for Open Source'
        >>> t.subtitle
        u'Helping to make the world a better place:  One project at a time'
        >>> t.copyright
        u'This document is public domain'
        >>> t.body
        u'<p>The body of the tag document.</p>\n'

    Default values (except slug)::

        >>> # Test the empty-file case.
        >>> t = TagParser()
        >>> t = t.parse_one('content/tags/empty.rst')
        >>> t.slug
        u'empty'
        >>> t.title
        u''
        >>> t.subtitle
        u''
        >>> t.copyright
        u''
        >>> t.body
        u''

    Note that posts are not cross-referenced at this point.  This also implies
    that the update time is not set::

        >>> t.posts
        Traceback (most recent call last):
        AttributeError: 'RstObject' object has no attribute 'posts'
        >>> t.updated
        Traceback (most recent call last):
        AttributeError: 'RstObject' object has no attribute 'updated'

    All tags may be retrieved with::

        >>> from pysettings.settings import Settings
        >>> s = {'CONTENT_ROOT': 'content'
        ...     ,'TAGS_SUBDIR': 'tags'
        ...     ,'REST_EXTENSION': 'rst'
        ...     }
        >>> s = Settings(s)
        >>> t = TagParser(s)
        >>> t.log = Mock('log')
        >>> map(lambda t: t.slug, t.parse())
        Called log.info('parsed content/tags/empty.rst')
        Called log.info('parsed content/tags/rcos.rst')
        [u'empty', u'rcos']

    '''

    auto_metadata = [('copyright', 'copyright')
                    ]

    auto_pubparts = [('title', 'title')
                    ,('subtitle', 'subtitle')
                    ,('body', 'fragment')
                    ]

    def paths(self):
        '''Return a list of paths to tag objects on the filesystem.

        Consider all the contents of the tags directory (by default this is
        ``content_root/tags``).  Only files ending in ``suffix`` are considered.

        Directory entries that are not files are ignored.

            >>> from pysettings.settings import Settings
            >>> s = {'CONTENT_ROOT': 'content'
            ...     ,'TAGS_SUBDIR': 'tags'
            ...     ,'REST_EXTENSION': 'rst'
            ...     }
            >>> s = Settings(s)
            >>> t = TagParser(s)
            >>> t.paths()
            ['content/tags/empty.rst', 'content/tags/rcos.rst']

        '''
        # TODO add above settings to global settings.
        settings = self.settings
        path  = os.path.join(settings.CONTENT_ROOT, settings.TAGS_SUBDIR)
        files = os.listdir(path)
        files = map(lambda p: os.path.join(path, p), files)
        files = filter(lambda f: f.endswith(settings.REST_EXTENSION), files)
        files = filter(lambda f: os.path.isfile(f), files)
        files.sort()
        return files

    def new_object(self, path, d, pub):
        '''Return an instance of the object to which rst documents are parsed.
        '''
        f = RstObject()
        f.slug = unicode(os.path.basename(path).rsplit('.', 1)[0])
        return f

    def default(self, attr):
        '''Return the default value of an attribute.
        '''
        return u''



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''The Firmant application.

Orchestrates the high process of parsing, transforming, and writing content.
'''


import logging
import datetime

from pysettings.modules import get_module

from firmant.i18n import _
from firmant.routing import URLMapper
from firmant.utils import class_name


class Firmant(object):
    '''Perform a complete run from parsing through writing.

        >>> from pysettings.loaders import mod_to_settings
        >>> s = mod_to_settings('firmant.settings')
        >>> s.OUTPUT_DIR = outdir
        >>> f = Firmant(s)
        >>> f.log = Mock('log')
        >>> pprint(f.parsers) #doctest: +ELLIPSIS
        {'feeds': <firmant.parsers.feeds.FeedParser object at 0x...>,
         'posts': <firmant.parsers.posts.PostParser object at 0x...>,
         'static': <firmant.parsers.static.StaticParser object at 0x...>,
         'tags': <firmant.parsers.tags.TagParser object at 0x...>}
        >>> f.parse()
        >>> pprint(f.objs) #doctest: +ELLIPSIS
        {'feeds': [<firmant.parsers.RstObject object at 0x...>,
                   <firmant.parsers.RstObject object at 0x...>,
                   <firmant.parsers.RstObject object at 0x...>,
                   <firmant.parsers.RstObject object at 0x...>],
         'posts': [<firmant.parsers.RstObject object at 0x...>,
                   <firmant.parsers.RstObject object at 0x...>,
                   <firmant.parsers.RstObject object at 0x...>,
                   <firmant.parsers.RstObject object at 0x...>,
                   <firmant.parsers.RstObject object at 0x...>],
         'static': [static_obj<testdata/pristine/static/images/88x31.png>],
         'tags': [<firmant.parsers.RstObject object at 0x...>,
                  <firmant.parsers.RstObject object at 0x...>,
                  <firmant.parsers.RstObject object at 0x...>,
                  <firmant.parsers.RstObject object at 0x...>]}
        >>> f.cross_reference() #doctest: +ELLIPSIS
        >>> for post in f.objs['posts']:
        ...     pprint((post.tags, post.feeds)) #doctest: +ELLIPSIS
        ([<firmant.parsers.RstObject object at 0x...>],
         [<firmant.parsers.RstObject object at 0x...>,
          <firmant.parsers.RstObject object at 0x...>,
          <firmant.parsers.RstObject object at 0x...>,
          <firmant.parsers.RstObject object at 0x...>])
        ([<firmant.parsers.RstObject object at 0x...>],
         [<firmant.parsers.RstObject object at 0x...>,
          <firmant.parsers.RstObject object at 0x...>])
        ([<firmant.parsers.RstObject object at 0x...>],
         [<firmant.parsers.RstObject object at 0x...>,
          <firmant.parsers.RstObject object at 0x...>])
        ([<firmant.parsers.RstObject object at 0x...>,
          <firmant.parsers.RstObject object at 0x...>],
         [<firmant.parsers.RstObject object at 0x...>,
          <firmant.parsers.RstObject object at 0x...>])
        ([<firmant.parsers.RstObject object at 0x...>,
          <firmant.parsers.RstObject object at 0x...>],
         [<firmant.parsers.RstObject object at 0x...>,
          <firmant.parsers.RstObject object at 0x...>])
        >>> f.setup_writers()
        >>> f.check_url_conflicts()
        >>> f.create_permalinks()
        Called log.warning('Object type tags has no permalink providers.')
        >>> pprint([post.permalink for post in f.objs['posts']])
        ['http://test/2009/12/31/party/index.html',
         'http://test/2010/01/01/newyear/index.html',
         'http://test/2010/02/01/newmonth/index.html',
         'http://test/2010/02/02/newday/index.html',
         'http://test/2010/02/02/newday2/index.html']
        >>> pprint([feed.permalink for feed in f.objs['feeds']])
        ['http://test/bar/index.atom',
         'http://test/baz/index.atom',
         'http://test/foo/index.atom',
         'http://test/quux/index.atom']
        >>> pprint([static.permalink for static in f.objs['static']])
        ['http://test/images/88x31.png']
        >>> f.write()

    '''

    def __init__(self, settings):
        self.settings = settings
        if hasattr(settings, 'PERMALINK_ROOT'):
            root = settings.PERMALINK_ROOT
            self.urlmapper = URLMapper(getattr(settings, 'URLS', None), root)
        else:
            self.urlmapper = URLMapper(getattr(settings, 'URLS', None))
        self.log = logging.getLogger(class_name(self.__class__))

        self.objs = dict()
        self.writers = dict()

        # Setup parsers
        self.parsers = dict()
        for key, parser in settings.PARSERS.items():
            mod, attr = parser.rsplit('.', 1)
            mod = get_module(mod)
            parser = getattr(mod, attr)
            self.parsers[key] = parser(self.settings)

    def parse(self):
        '''Call parse on each configured parser.
        '''
        # Parse documents
        for key, parser in self.parsers.items():
            self.objs[key] = parser.parse()

    def cross_reference(self):
        '''Cross reference tags and feeds
        '''
        for feed in self.objs.get('feeds', []):
            feed.posts = list()
        for tag in self.objs.get('tags', []):
            tag.posts = list()
        # TODO inefficient, notproud
        for post in self.objs.get('posts', []):
            to_delete = list()
            for i, ptag in enumerate(post.tags):
                seen = False
                for tag in self.objs.get('tags', []):
                    if tag.slug == ptag:
                        seen = True
                        tag.posts.append(post)
                        post.tags[i] = tag
                if not seen:
                    warning  = _("Tag '%s' referenced but not defined.")
                    warning %= ptag
                    self.log.warning(warning)
                    to_delete.append(i)
            for i in reversed(to_delete):
                del post.tags[i]
            to_delete = list()
            for i, pfeed in enumerate(post.feeds):
                seen = False
                for feed in self.objs.get('feeds', []):
                    if feed.slug == pfeed:
                        seen = True
                        feed.posts.append(post)
                        post.feeds[i] = feed
                if not seen:
                    error  = _("Feed '%s' referenced but not defined.")
                    error %= pfeed
                    self.log.error(error)
                    to_delete.append(i)
            for i in reversed(to_delete):
                del post.feeds[i]

    def setup_writers(self):
        '''Create instances of writer classes.
        '''
        writers = self.writers
        for writer in self.settings.WRITERS:
            mod, attr = writer.rsplit('.', 1)
            mod = get_module(mod)
            writer = getattr(mod, attr)
            # We do not retain the class name from settings as it may be the
            # case that:
            # from foo import Quux
            # really imports the class foo.bar.baz.Quux as Quux was imported
            # into the foo namespace.
            writers[class_name(writer)] = writer(self.settings, self.objs,
                    self.urlmapper)

    def check_url_conflicts(self):
        '''Create instances of writer classes.
        '''
        urls = dict()
        for key, inst in self.writers.items():
            for url in inst.urls():
                if url is None:
                    warning  = _("Writer %s's URLs incompletely defined.")
                    warning %= key
                    self.log.warning(warning)
                elif url in urls:
                    warning  = _('Writers %s and %s conflict over %s.')
                    warning %= urls[url], key, url
                    self.log.warning(warning)
                else:
                    urls[url] = key

    def create_permalinks(self):
        '''Add a 'permalink' attribute to each parsed object.
        '''
        permalink_providers = dict()
        for inst in self.writers.values():
            if hasattr(inst, 'permalinks_for'):
                p_for = inst.permalinks_for
                if p_for in permalink_providers:
                    warning  = _("Object type '%s' has multiple "
                                 "permalink providers.")
                    warning %= p_for
                    self.log.warning(warning)
                else:
                    permalink_providers[p_for] = inst.url
        for obj_tag, obj_list in self.objs.items():
            if obj_tag in permalink_providers:
                url_for = permalink_providers[obj_tag]
                for obj in obj_list:
                    setattr(obj, 'permalink', url_for(obj))
            else:
                warning  = _("Object type %s has no permalink providers.")
                warning %= obj_tag
                self.log.warning(warning)

    def create_globals(self):
        '''Create a dictionary of globals to be added to rendering contexts.
        '''
        self.objs['globals'] = globals = dict()
        urlfor = self.urlmapper.urlfor
        globals['urlfor'] = lambda fmt, **kwargs: \
            urlfor(fmt, absolute=True, **kwargs)
        globals['recent_posts'] = [(p.title, p.permalink) for p in
                reversed(sorted(self.objs.get('posts', []),
                    key=lambda p: (p.published.date(), p.slug)))] \
                [:self.settings.SIDEBAR_POSTS_LEN]
        # TODO:  Disgusted with this.  I will make a real global object later.
        globals['daily_archives'] = \
                [(datetime.date(y, m, d), urlfor('html', absolute=True,
                    type='post', year=y, month=m, day=d, page=1))
                for y, m, d in reversed(sorted(set([(p.published.year,
                    p.published.month, p.published.day)
                    for p in self.objs.get('posts', [])])))] \
                [:self.settings.SIDEBAR_ARCHIVES_LEN]
        globals['monthly_archives'] = \
                [(datetime.date(y, m, 1), urlfor('html', absolute=True,
                    type='post', year=y, month=m, page=1))
                for y, m in reversed(sorted(set([(p.published.year, p.published.month)
                    for p in self.objs.get('posts', [])])))] \
                [:self.settings.SIDEBAR_ARCHIVES_LEN]
        globals['yearly_archives'] = \
                [(datetime.date(y, 1, 1), urlfor('html', absolute=True,
                    type='post', year=y, page=1))
                for y in reversed(sorted(set([p.published.year
                    for p in self.objs.get('posts', [])])))] \
                [:self.settings.SIDEBAR_ARCHIVES_LEN]

    def write(self):
        '''Call ``write`` on each writer.
        '''
        for writer in self.writers.itervalues():
            writer.write()


def _setup(self):
    '''Setup the test cases.

    Actions taken::

        - Create a temporary directory.

    '''
    import tempfile
    self.globs['outdir'] = tempfile.mkdtemp()


def _teardown(test):
    '''Cleanup the test cases.

    Actions taken::

        - Remove the temporary directory.

    '''
    import shutil
    shutil.rmtree(test.globs['outdir'])



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''Copy static objects into their proper place in the output.
'''


import os
import shutil

from firmant.writers import Writer


class StaticWriter(Writer):
    '''Parse the posts into single feed documents.
    '''

    permalinks_for = 'static'

    def path(self, static):
        '''Use the urlmapper to construct a path for the given attributes.
        '''
        urlfor = self.urlmapper.urlfor
        return urlfor('', static=static.relpath)

    def url(self, static):
        '''Use the urlmapper to construct a URL for the given attributes.
        '''
        urlfor = self.urlmapper.urlfor
        return urlfor('', absolute=True, static=static.relpath)

    def urls(self):
        '''A list of rooted paths that are the path component of URLs.

        Example on testdata/pristine::

            >>> from firmant import routing
            >>> urlmapper.add(routing.SinglePathComponent('static', str))
            >>> sw = StaticWriter(settings, objs, urlmapper)
            >>> pprint(sw.urls())
            ['images/88x31.png']

        '''
        ret = list()
        for static in self.objs.get('static', []):
            ret.append(self.path(static))
        ret.sort()
        return ret

    def write(self):
        '''Write a parsed feed to the filesystem.
        '''
        for static in self.objs.get('static', []):
            relpath = self.path(static)
            abspath = os.path.join(self.settings.OUTPUT_DIR, relpath)
            try:
                os.makedirs(os.path.dirname(abspath))
            except OSError, e:
                if e.errno != 17:
                    raise
            shutil.copy2(static.fullpath, abspath)


def _setup(self):
    '''Setup the test cases.

    Actions taken::

        - Create a ``Settings`` object.
        - Create a ``Firmant`` object.
        - Load modules used in tests.

    '''
    from pysettings.settings import Settings
    from firmant.application import Firmant
    from firmant.routing import URLMapper
    from firmant.routing import components
    s = {'PARSERS': {'posts': 'firmant.parsers.posts.PostParser'
                    ,'feeds': 'firmant.parsers.feeds.FeedParser'
                    ,'static': 'firmant.parsers.static.StaticParser'
                    ,'tags': 'firmant.parsers.tags.TagParser'
                    }
        ,'CONTENT_ROOT': 'testdata/pristine'
        ,'POSTS_SUBDIR': 'posts'
        ,'FEEDS_SUBDIR': 'feeds'
        ,'TAGS_SUBDIR': 'feeds'
        ,'STATIC_SUBDIR': 'static'
        ,'REST_EXTENSION': 'rst'
        ,'POSTS_PER_PAGE': 2
        }
    settings               = Settings(s)
    firmant                = Firmant(settings)
    from minimock import Mock
    firmant.parse()
    firmant.cross_reference()
    self.globs['settings'] = settings
    self.globs['objs']  = firmant.objs
    self.globs['urlmapper'] = URLMapper()
    self.globs['components'] = components



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import logging
import stat
import tempfile
import os

from docutils import io
from docutils.core import publish_programmatically

from firmant.parsers import Parser
from firmant.du import MetaDataStandaloneReader
from firmant.i18n import _
from firmant.utils import class_name
from firmant.utils.exceptions import log_uncaught_exceptions


__all__ = ['StaticParser']


class StaticObject(object):
    '''An object that serves as a placeholder for a file on the filesystem.
    '''

    __slots__ = ['fullpath', 'relpath', 'permalink']

    def __repr__(self):
        return 'static_obj<%s>' % self.fullpath


class StaticParser(Parser):
    '''Create stand-in objects for static files to be published.

    '''

    def paths(self):
        '''Return a list of paths to objects on the file system.
        '''
        settings = self.settings
        path  = os.path.join(settings.CONTENT_ROOT, settings.STATIC_SUBDIR)
        all_files = []
        for root, dirs, files in os.walk(path):
            for file in files:
                s = StaticObject()
                s.fullpath = os.path.join(root, file)
                if s.fullpath.startswith(path):
                    s.relpath = s.fullpath[len(path):]
                else:
                    raise RuntimeError('Funky paths in StaticParser')
                if s.relpath.startswith('/'):
                    s.relpath = s.relpath[1:]
                all_files.append(s)
        all_files.sort(key=lambda s: s.relpath)
        return all_files

    def parse_one(self, path):
        '''We use the identity function simply because we are copying files.
        '''
        return path



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''Utilities for handling or silencing exceptions.
'''


import os
import tempfile
import traceback

from firmant.i18n import _


def log_uncaught_exceptions(func, log, message, save_traceback=False):
    '''Catch and log exceptions of ``func``.

    Returns True if the function succeeded without throwing an exception.

    A function for safely calling code that may throw exceptions and logging any
    exceptions thrown.  Optionally it will save the traceback to a file::

        >>> def raises_error():
        ...     raise RuntimeError('Intentionally thrown')
        >>> log_uncaught_exceptions(raises_error, Mock('log'), 'error!')
        Called log.error('error!')
        Called log.info('traceback not saved')
        False

    If ``save_traceback`` is True, it will save the file to one created with
    ``tempfile.mkstemp``.

        >>> from minimock import restore
        >>> import tempfile
        >>> f, path = tempfile.mkstemp()
        >>> tempfile.mkstemp = Mock('mkstemp')
        >>> tempfile.mkstemp.mock_returns = (f, path)
        >>> def raises_error():
        ...     raise RuntimeError('Intentionally thrown')
        >>> log_uncaught_exceptions(raises_error, Mock('log'), 'error!', True) #doctest: +ELLIPSIS
        Called log.error('error!')
        Called mkstemp(prefix='firmant', text=True)
        Called log.error(...'traceback saved to /...')
        False
        >>> os.unlink(path)
        >>> restore()

    If an exception is thrown while saving to the file, it will warn about
    the potential for infinite recursion and stop::

        >>> from minimock import restore
        >>> import tempfile
        >>> tempfile.mkstemp = Mock('mkstemp')
        >>> tempfile.mkstemp.mock_raises = ValueError
        >>> def raises_error():
        ...     raise RuntimeError('Intentionally thrown')
        >>> log_uncaught_exceptions(raises_error, Mock('log'), 'error!', True)
        Called log.error('error!')
        Called mkstemp(prefix='firmant', text=True)
        Called log.error("it's turtles all the way down")
        False
        >>> restore()

    '''
    try:
        func()
    except:
        log.error(message)
        if save_traceback:
            try:
                t, path = tempfile.mkstemp(prefix='firmant', text=True)
                t = os.fdopen(t, 'w+')
                traceback.print_exc(file=t)
                t.flush()
                t.close()
                log.error(_('traceback saved to %s') % path)
            except:
                import sys
                log.error(_("it's turtles all the way down"))
        else:
            log.info(_('traceback not saved'))
    else:
        return True
    return False



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import datetime
import os
import sys


def class_name(cls):
    '''Return a string representation of a class's name.

        >>> class Foo(object): pass
        ...
        >>> class_name(Foo)
        'firmant.utils.Foo'

    '''
    return str(cls)[8:-2]


def strptime(date_string, formats):
    '''Cast the date_string to the first format to match.

    Examples::

        >>> strptime('2009-02-01 11:51:15', ['%Y-%m-%d %H:%M:%S'])
        datetime.datetime(2009, 2, 1, 11, 51, 15)
        >>> strptime('11:51:15', ['%H:%M:%S'])
        datetime.datetime(1900, 1, 1, 11, 51, 15)

    If the time does not match, a value error will be raised::

        >>> strptime('AB:CD:EF', ['%H:%M:%S'])
        Traceback (most recent call last):
        ValueError: time data 'AB:CD:EF' does not match any format.

    '''
    error = None
    dt    = None
    for format in formats:
        try:
            dt = dt or datetime.datetime.strptime(date_string, format)
        except ValueError:
            pass
    if dt is None:
        error = "time data '%s' does not match any format." % date_string
        raise ValueError(error)
    return dt


def merge_dicts(a, *args):
    '''Merge two dictionaries.  Return the merge.

    Where identical keys correspond to conflicting values, a ValueError is
    rasied.

        >>> pprint(merge_dicts({'a': 1}))
        {'a': 1}
        >>> pprint(merge_dicts({'a': 1}, {'b': 2}))
        {'a': 1, 'b': 2}
        >>> pprint(merge_dicts({'a': 1, 'b': 2}, {'b': 2}))
        {'a': 1, 'b': 2}

    More than two dictionaries may be merged as well::

        >>> pprint(merge_dicts({'a': 1, 'b': 2}, {'b': 2}, {'c':3}))
        {'a': 1, 'b': 2, 'c': 3}

    Error conditions::

        >>> pprint(merge_dicts({'a': 1, 'b': 2}, {'b': 3}))
        Traceback (most recent call last):
        ValueError: Conflicting values for 'b'

    '''
    ret = a.copy()
    for b in args:
        conflicts = set(a.keys()) & set(b.keys())
        for key in conflicts:
            if key in a and key in b and a[key] != b[key]:
                raise ValueError("Conflicting values for '%s'" % key)
        ret.update(b)
    return ret



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''Functions for interacting with the docutils module.
'''


__all__ = ['Copyright', 'publish_parts_doc']


from docutils.parsers.rst import Directive
from docutils.parsers.rst import directives
from docutils.transforms import Transform
from docutils.readers import standalone
from docutils import nodes

from firmant.utils import strptime


class MetaDataNode(nodes.Special, nodes.Invisible, nodes.Element):
    '''The MetaDataNode is a node that holds metadata for later Transforms.
    '''

    def __init__(self, details=None, rawsource='', *children, **attributes):
        nodes.Element.__init__(self, rawsource, *children, **attributes)

        self.details = details or dict()


def meta_data_directive(func, whitespace=False):
    '''Create a Directive class to store data to a MetaDataNode.

    The function ``func`` is passed the contents of the node and a dictionary
    into which it should place all relevant metadata it extracts from the
    contents.
    '''

    class MetaDataDirective(Directive):
        '''A generic directive for capturing metadata.
        '''

        required_arguments = 0
        optional_arguments = 0
        final_argument_whitespace = whitespace
        option_spec = dict()
        has_content = True

        def run(self):
            # Raise an error if the directive does not have contents.
            self.assert_has_content()

            # Parse the contents into metadata.
            d = dict()
            try:
                func(d, self.content)
            except ValueError, e:
                error = self.state_machine.reporter.error(str(e))
                return []

            # Insert a MetaDataNode with the metadata.
            node = MetaDataNode(d)
            return [node]

    return MetaDataDirective


def copyright(d, content):
    r'''Interpret the content as a copyright declaration.

        >>> d = dict()
        >>> copyright(d, [u'foo', u'bar'])
        >>> d['copyright']
        u'foo\nbar'

    '''
    d['copyright'] = '\n'.join(content)


def time(d, content):
    '''Interpret the content as a time.

    Using just hours::

        >>> d = dict()
        >>> time(d, ['15'])
        >>> d['time']
        datetime.time(15, 0)

    Using hours and minutes::

        >>> d = dict()
        >>> time(d, ['15:43'])
        >>> d['time']
        datetime.time(15, 43)

    Using hours, minutes, and seconds::

        >>> d = dict()
        >>> time(d, ['15:43:42'])
        >>> d['time']
        datetime.time(15, 43, 42)

    ValueError is raised on invalid time::

        >>> d = dict()
        >>> time(d, ['154342'])
        Traceback (most recent call last):
        ValueError: time data '154342' does not match any format.

    '''
    formats = ['%H', '%H:%M', '%H:%M:%S']
    dt = strptime(''.join(content), formats)
    d['time'] = dt.time()


def single_line(d, content, attr='line'):
    '''Interpret the content as a single line.

    Default behavior::

        >>> d = dict()
        >>> single_line(d, [u'foobar'])
        >>> d['line']
        u'foobar'

    Specifying the attribute to save to::

        >>> d = dict()
        >>> single_line(d, [u'foobar'], 'attr')
        >>> d['attr']
        u'foobar'

    '''
    d[attr] = unicode(''.join(content))


def updated(d, content):
    '''Interpret content as a full datetime.

    YYYY-MM-DD HH:MM:SS (time may be omitted)::

        >>> d = dict()
        >>> updated(d, ['2009-02-01 14:15:51'])
        >>> d['updated']
        datetime.datetime(2009, 2, 1, 14, 15, 51)

    MM-DD-YYYY HH:MM:SS (time may be omitted)::

        >>> d = dict()
        >>> updated(d, ['02-01-2009 14:15:51'])
        >>> d['updated']
        datetime.datetime(2009, 2, 1, 14, 15, 51)

    ValueError is raised on invalid datetime::

        >>> d = dict()
        >>> updated(d, ['154342'])
        Traceback (most recent call last):
        ValueError: time data '154342' does not match any format.

    '''
    formats = ['%Y-%m-%d', '%Y-%m-%d %H', '%Y-%m-%d %H:%M',
               '%Y-%m-%d %H:%M:%S', '%m-%d-%Y', '%m-%d-%Y %H',
               '%m-%d-%Y %H:%M', '%m-%d-%Y %H:%M:%S']
    d['updated'] = strptime(''.join(content), formats)


def list_element(d, content, attr='element_plural'):
    '''Interpret content as an element in a list.

    MetaData of this type may have several directives, each of which appends a
    value to list.

    It returns a list so that the metadata transforms may simply append tags::

        >>> d = dict()
        >>> list_element(d, [u'foo'])
        >>> d['element_plural']
        [u'foo']

    An attribute for storing values may be provided::

        >>> d = dict()
        >>> list_element(d, [u'foo'], 'attr')
        >>> d['attr']
        [u'foo']

    '''
    d[attr] = list([''.join(content)])


_Copyright = meta_data_directive(copyright, whitespace=True)
directives.register_directive('copyright', _Copyright)

_Time = meta_data_directive(time)
directives.register_directive('time', _Time)

_Timezone = meta_data_directive(lambda d, c: single_line(d, c, 'timezone'))
directives.register_directive('timezone', _Timezone)

_Author = meta_data_directive(lambda d, c: single_line(d, c, 'author'))
directives.register_directive('author', _Author)

_Updated = meta_data_directive(updated)
directives.register_directive('updated', _Updated)

_Tag = meta_data_directive(lambda d, c: list_element(d, c, 'tags'))
directives.register_directive('tag', _Tag)

_Feed = meta_data_directive(lambda d, c: list_element(d, c, 'feeds'))
directives.register_directive('feed', _Feed)


def meta_data_transform(data):

    class MetaDataTransform(Transform):
        '''Remove MetaDataNode nodes and set attributes on document.
        '''
        default_priority = 700

        def apply(self):
            for node in self.document.traverse(MetaDataNode):
                for key, val in node.details.items():
                    try:
                        iter(val)
                        if key in data:
                            val = data[key] + val
                    except TypeError:
                        pass
                    data[key] = val
                node.parent.remove(node)
    return MetaDataTransform


class MetaDataStandaloneReader(standalone.Reader):
    '''Add transformations to read MetaData from doctree.
    '''

    def __init__(self, parser=None, parser_name=None, data=None):
        standalone.Reader.__init__(self, parser, parser_name)
        if data is None:
            data = dict()
        self.data = data

    def get_transforms(self):
        return standalone.Reader.get_transforms(self) + \
            [meta_data_transform(self.data)]



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''Predefined path components for use with URLMapper.
'''


from firmant import routing


__all__ = ['Type', 'Year', 'Month', 'Day', 'Slug', 'PageNo', 'year', 'month',
        'day', 'slug', 'pageno']


# Classes


class Type(routing.BoundNullPathComponent):
    '''The type of page that is to be matched.

    For instance, a type of ``post`` will only match pages that declare
    themselves to render posts.

    Example::

        >>> Type('post').match(type='post')
        True
        >>> Type('post').match(type='tag')
        False

    '''

    def __init__(self, value):
        super(Type, self).__init__('type', value)


class Year(routing.SinglePathComponent):
    '''A year associated with the page to be matched.

    Example::

        >>> Year().construct(year=2010)
        '2010'

    '''

    @classmethod
    def _conv(cls, value):
        return '%04i' % value

    def __init__(self):
        super(Year, self).__init__('year', self._conv)


class Month(routing.SinglePathComponent):
    '''A month associated with the page to be matched.

    Example::

        >>> Month().construct(month=3)
        '03'

    '''

    @classmethod
    def _conv(cls, value):
        return '%02i' % value

    def __init__(self):
        super(Month, self).__init__('month', self._conv)


class Day(routing.SinglePathComponent):
    '''A day associated with the page to be matched.

    Example::

        >>> Day().construct(day=15)
        '15'

    '''

    @classmethod
    def _conv(cls, value):
        return '%02i' % value

    def __init__(self):
        super(Day, self).__init__('day', self._conv)


class Slug(routing.SinglePathComponent):
    '''The slug associated with the page to be matched.

    Example::

        >>> Slug().construct(slug='foobar')
        'foobar'

    '''

    @classmethod
    def _conv(cls, value):
        return str(value)

    def __init__(self):
        super(Slug, self).__init__('slug', self._conv)


class PageNo(routing.SinglePathComponent):
    '''The page number associated with the page to be matched.

    Example::

        >>> PageNo().construct(page=1) is None
        True
        >>> PageNo().construct(page=2)
        'page2'

    '''

    @classmethod
    def _conv(cls, value):
        if value == 1:
            return None
        return 'page%i' % value

    def __init__(self):
        super(PageNo, self).__init__('page', self._conv)


# Convenient instances of all classes not tied to settings.


year    = Year()
month   = Month()
day     = Day()
slug    = Slug()
pageno  = PageNo()



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''Writers that use the Jinja2 templating engine to create html.
'''


import os

from jinja2 import Environment
from jinja2 import FileSystemLoader

from firmant import utils
from firmant.utils import paths
from firmant.writers import posts
from firmant.writers import Writer


class Jinja2Writer(Writer):
    '''Base class used for functionality common to all J2 writers.
    '''

    def __init__(self, *args, **kwargs):
        super(Jinja2Writer, self).__init__(*args, **kwargs)
        loader = FileSystemLoader(self.settings.TEMPLATE_DIR)
        self._env = Environment(loader=loader)

    def render_to_file(self, path, template, context):
        '''Render template with context and save to path.
        '''
        template = self._env.get_template(template)
        globals  = self.objs.get('globals', dict())
        globals.update(context)
        data     = template.render(globals)
        path     = os.path.join(self.settings.OUTPUT_DIR, path or '')
        f        = paths.create_or_truncate(path)
        f.write(data.encode('utf-8'))
        f.flush()
        f.close()


class Jinja2PostArchiveBase(Jinja2Writer, posts.PostArchiveBase):
    '''Common functionality for rendering Jinja2 archive views.
    '''

    def render_common(self, post_list, cal, pages):
        '''Return the url, template, and context, ready to render.
        '''
        rev      = self.rev_key(cal.cur)
        url      = self.path(page=pages.cur, **rev)
        template = self.template
        context  = dict()
        context.update(rev)
        context['posts'] = post_list
        context['pprev'] = pages.prev and \
                self.path(absolute=True, page=pages.prev, **rev)
        context['pnext'] = pages.next and \
                self.path(absolute=True, page=pages.next, **rev)
        context['sprev'] = cal.prev and \
                self.path(absolute=True, page=1, **self.rev_key(cal.prev))
        context['snext'] = cal.next and \
                self.path(absolute=True, page=1, **self.rev_key(cal.next))
        return url, template, context


class Jinja2PostArchiveAll(Jinja2PostArchiveBase, posts.PostArchiveAll):
    '''Render paginated post lists with Jinja2 templates.
    '''

    fmt = 'html'

    template = 'posts/archive_all.html'

    def render(self, post_list, pages):
        r'''Render the data in a Jinja2 template.

            >>> c = components
            >>> urlmapper.add(c.Type('post')/c.pageno)
            >>> j2paa = Jinja2PostArchiveAll(settings, objs, urlmapper)
            >>> j2paa.log = Mock('log')
            >>> j2paa.write()
            >>> cat(os.path.join(settings.OUTPUT_DIR, 'index.html'))
            Called stdout.write('Prev: \n')
            Called stdout.write('Next: http://test/page2/index.html\n')
            Called stdout.write('2010-02-02-newday2\n')
            Called stdout.write('2010-02-02-newday\n')
            >>> cat(os.path.join(settings.OUTPUT_DIR, 'page2/index.html'))
            Called stdout.write('Prev: http://test/index.html\n')
            Called stdout.write('Next: http://test/page3/index.html\n')
            Called stdout.write('2010-02-01-newmonth\n')
            Called stdout.write('2010-01-01-newyear\n')
            >>> cat(os.path.join(settings.OUTPUT_DIR, 'page3/index.html'))
            Called stdout.write('Prev: http://test/page2/index.html\n')
            Called stdout.write('Next: \n')
            Called stdout.write('2009-12-31-party\n')

        '''
        class EmptyCal(object):
            def __init__(self):
                self.prev = None
                self.cur  = None
                self.next = None
        empty = EmptyCal()

        url, template, context = self.render_common(post_list, empty, pages)
        self.render_to_file(url, template, context)


class Jinja2PostArchiveYearly(Jinja2PostArchiveBase, posts.PostArchiveYearly):
    '''Render paginated post lists (grouped by year) with Jinja2 templates.
    '''

    fmt = 'html'

    template = 'posts/archive_yearly.html'

    def render(self, post_list, years, pages):
        r'''Render the data in a Jinja2 template.

            >>> c = components
            >>> urlmapper.add(c.Type('post')/c.year/c.pageno)
            >>> j2pay = Jinja2PostArchiveYearly(settings, objs, urlmapper)
            >>> j2pay.log = Mock('log')
            >>> j2pay.write()
            >>> cat(os.path.join(settings.OUTPUT_DIR, '2010/index.html'))
            Called stdout.write('Prev year: \n')
            Called stdout.write('Next year: http://test/2009/index.html\n')
            Called stdout.write('Prev page: \n')
            Called stdout.write('Next page: http://test/2010/page2/index.html\n')
            Called stdout.write('2010-02-02-newday2\n')
            Called stdout.write('2010-02-02-newday\n')
            >>> cat(os.path.join(settings.OUTPUT_DIR, '2010/page2/index.html'))
            Called stdout.write('Prev year: \n')
            Called stdout.write('Next year: http://test/2009/index.html\n')
            Called stdout.write('Prev page: http://test/2010/index.html\n')
            Called stdout.write('Next page: \n')
            Called stdout.write('2010-02-01-newmonth\n')
            Called stdout.write('2010-01-01-newyear\n')
            >>> cat(os.path.join(settings.OUTPUT_DIR, '2009/index.html'))
            Called stdout.write('Prev year: http://test/2010/index.html\n')
            Called stdout.write('Next year: \n')
            Called stdout.write('Prev page: \n')
            Called stdout.write('Next page: \n')
            Called stdout.write('2009-12-31-party\n')

        '''
        url, template, context = self.render_common(post_list, years, pages)
        self.render_to_file(url, template, context)


class Jinja2PostArchiveMonthly(Jinja2PostArchiveBase, posts.PostArchiveMonthly):
    '''Render paginated post lists (grouped by month) with Jinja2 templates.
    '''

    fmt = 'html'

    template = 'posts/archive_monthly.html'

    def render(self, post_list, months, pages):
        r'''Render the data in a Jinja2 template.

            >>> c = components
            >>> urlmapper.add(
            ...     c.Type('post')/c.year/c.month/c.pageno)
            >>> j2pam = Jinja2PostArchiveMonthly(settings, objs, urlmapper)
            >>> j2pam.log = Mock('log')
            >>> j2pam.write()
            >>> cat(os.path.join(settings.OUTPUT_DIR, '2010/02/index.html'))
            Called stdout.write('Prev month: \n')
            Called stdout.write('Next month: http://test/2010/01/index.html\n')
            Called stdout.write('Prev page: \n')
            Called stdout.write('Next page: http://test/2010/02/page2/index.html\n')
            Called stdout.write('2010-02-02-newday2\n')
            Called stdout.write('2010-02-02-newday\n')
            >>> cat(os.path.join(settings.OUTPUT_DIR, '2010/02/page2/index.html'))
            Called stdout.write('Prev month: \n')
            Called stdout.write('Next month: http://test/2010/01/index.html\n')
            Called stdout.write('Prev page: http://test/2010/02/index.html\n')
            Called stdout.write('Next page: \n')
            Called stdout.write('2010-02-01-newmonth\n')
            >>> cat(os.path.join(settings.OUTPUT_DIR, '2010/01/index.html'))
            Called stdout.write('Prev month: http://test/2010/02/index.html\n')
            Called stdout.write('Next month: http://test/2009/12/index.html\n')
            Called stdout.write('Prev page: \n')
            Called stdout.write('Next page: \n')
            Called stdout.write('2010-01-01-newyear\n')
            >>> cat(os.path.join(settings.OUTPUT_DIR, '2009/12/index.html'))
            Called stdout.write('Prev month: http://test/2010/01/index.html\n')
            Called stdout.write('Next month: \n')
            Called stdout.write('Prev page: \n')
            Called stdout.write('Next page: \n')
            Called stdout.write('2009-12-31-party\n')

        '''
        url, template, context = self.render_common(post_list, months, pages)
        self.render_to_file(url, template, context)


class Jinja2PostArchiveDaily(Jinja2PostArchiveBase, posts.PostArchiveDaily):
    '''Render paginated post lists (grouped by day) with Jinja2 templates.
    '''

    fmt = 'html'

    template = 'posts/archive_daily.html'

    def render(self, post_list, days, pages):
        r'''Render the data in a Jinja2 template.

            >>> c = components
            >>> settings.POSTS_PER_PAGE = 1
            >>> urlmapper.add(
            ...     c.Type('post')/c.year/c.month/c.day/c.pageno)
            >>> j2pad = Jinja2PostArchiveDaily(settings, objs, urlmapper)
            >>> j2pad.log = Mock('log')
            >>> j2pad.write()
            >>> cat(os.path.join(settings.OUTPUT_DIR, '2010/02/02/index.html'))
            Called stdout.write('Prev day: \n')
            Called stdout.write('Next day: http://test/2010/02/01/index.html\n')
            Called stdout.write('Prev page: \n')
            Called stdout.write('Next page: http://test/2010/02/02/page2/index.html\n')
            Called stdout.write('2010-02-02-newday2\n')
            >>> cat(os.path.join(settings.OUTPUT_DIR, '2010/02/02/page2/index.html'))
            Called stdout.write('Prev day: \n')
            Called stdout.write('Next day: http://test/2010/02/01/index.html\n')
            Called stdout.write('Prev page: http://test/2010/02/02/index.html\n')
            Called stdout.write('Next page: \n')
            Called stdout.write('2010-02-02-newday\n')
            >>> cat(os.path.join(settings.OUTPUT_DIR, '2010/02/01/index.html'))
            Called stdout.write('Prev day: http://test/2010/02/02/index.html\n')
            Called stdout.write('Next day: http://test/2010/01/01/index.html\n')
            Called stdout.write('Prev page: \n')
            Called stdout.write('Next page: \n')
            Called stdout.write('2010-02-01-newmonth\n')
            >>> cat(os.path.join(settings.OUTPUT_DIR, '2010/01/01/index.html'))
            Called stdout.write('Prev day: http://test/2010/02/01/index.html\n')
            Called stdout.write('Next day: http://test/2009/12/31/index.html\n')
            Called stdout.write('Prev page: \n')
            Called stdout.write('Next page: \n')
            Called stdout.write('2010-01-01-newyear\n')
            >>> cat(os.path.join(settings.OUTPUT_DIR, '2009/12/31/index.html'))
            Called stdout.write('Prev day: http://test/2010/01/01/index.html\n')
            Called stdout.write('Next day: \n')
            Called stdout.write('Prev page: \n')
            Called stdout.write('Next page: \n')
            Called stdout.write('2009-12-31-party\n')

        '''
        url, template, context = self.render_common(post_list, days, pages)
        self.render_to_file(url, template, context)


class Jinja2PostSingle(Jinja2Writer, posts.PostSingle):
    '''Render each post using Jinja2 templates.
    '''

    fmt = 'html'

    permalinks_for = 'posts'

    def render(self, post):
        r'''Render the data in a Jinja2 template.

            >>> c = components
            >>> urlmapper.add(c.Type('post')/c.year/c.month/c.day/c.slug)
            >>> j2ps = Jinja2PostSingle(settings, objs, urlmapper)
            >>> j2ps.log = Mock('log')
            >>> j2ps.write()
            >>> join = os.path.join
            >>> cat(join(settings.OUTPUT_DIR, '2010/02/02/newday2/index.html'))
            Called stdout.write('2010-02-02 | newday2 by John Doe')
            >>> cat(join(settings.OUTPUT_DIR, '2010/02/02/newday/index.html'))
            Called stdout.write('2010-02-02 | newday by John Doe')
            >>> cat(join(settings.OUTPUT_DIR, '2010/02/01/newmonth/index.html'))
            Called stdout.write('2010-02-01 | newmonth by John Doe')
            >>> cat(join(settings.OUTPUT_DIR, '2010/01/01/newyear/index.html'))
            Called stdout.write('2010-01-01 | newyear by John Doe')
            >>> cat(join(settings.OUTPUT_DIR, '2009/12/31/party/index.html'))
            Called stdout.write('2009-12-31 | party by John Doe')

        '''
        url = self.path(post=post)
        template = 'posts/single.html'
        context = dict()
        context['year']          = post.published.year
        context['month']         = post.published.month
        context['day']           = post.published.day
        context['slug']          = post.slug
        context['post']          = post
        self.render_to_file(url, template, context)


def _setup(self):
    '''Setup the test cases.

    Actions taken::

        - Create a temporary directory.
        - Create a ``Settings`` object.
        - Create a ``Firmant`` object.
        - Load modules used in tests.

    '''
    import tempfile
    from minimock import Mock

    from pysettings.settings import Settings
    from firmant.application import Firmant
    from firmant.routing import URLMapper
    from firmant.routing import components
    from firmant.utils.paths import cat
    s = {'PARSERS': {'posts': 'firmant.parsers.posts.PostParser'}
        ,'CONTENT_ROOT': 'testdata/pristine'
        ,'POSTS_SUBDIR': 'posts'
        ,'REST_EXTENSION': 'rst'
        ,'POSTS_PER_PAGE': 2
        ,'OUTPUT_DIR': tempfile.mkdtemp()
        ,'TEMPLATE_DIR': 'testdata/pristine/templates'
        ,'PERMALINK_ROOT': 'http://test'
        }
    settings               = Settings(s)
    firmant                = Firmant(settings)
    firmant.parse()
    self.globs['settings']   = settings
    self.globs['objs']       = firmant.objs
    self.globs['urlmapper']  = URLMapper(root=settings.PERMALINK_ROOT)
    self.globs['Mock']       = Mock
    self.globs['components'] = components
    self.globs['cat']        = lambda out: cat(out, Mock('stdout'))


def _teardown(test):
    '''Cleanup the Jinja2 test cases.
    '''
    import shutil
    shutil.rmtree(test.globs['settings'].OUTPUT_DIR)



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''Writers that write various views of the posts.
'''


from copy import copy

from firmant import paginate
from firmant.i18n import _
from firmant.writers import Writer


class PostWriter(Writer):
    '''Base class used for functionality common to all post writers.
    '''

    def _sorted_posts(self):
        '''Return the posts sorted by (date, slug)
        '''
        posts = copy(self.objs['posts'])
        posts.sort(key=lambda p: (p.published, p.slug), reverse=True)
        return posts


class PostArchiveBase(PostWriter):
    '''Parse the posts into lists of size POSTS_PER_PAGE.
    '''

    def write_preconditions(self):
        '''Returns true if and only if it is acceptable to proceed with writing.

        It is a precondition for writing that ``POSTS_PER_PAGE`` be set to a
        positive, non-zero integer.

            >>> from pysettings.settings import Settings
            >>> paa = PostArchiveAll(Settings(POSTS_PER_PAGE=0), {}, None)
            >>> paa.log = Mock('log')
            >>> paa.write_preconditions()
            Called log.critical('``POSTS_PER_PAGE`` must be a positive int.')
            False

        It is another precondition for writing that there be posts in the parsed
        objects dictionary::

            >>> from pysettings.settings import Settings
            >>> paa = PostArchiveAll(Settings(POSTS_PER_PAGE=1), {}, None)
            >>> paa.log = Mock('log')
            >>> paa.write_preconditions()
            Called log.critical('``posts`` must be in parsed objects.')
            False

        '''
        # Fail if we do not have an output directory.
        if getattr(self.settings, 'POSTS_PER_PAGE', -1) < 1:
            self.log.critical(_('``POSTS_PER_PAGE`` must be a positive int.'))
            return False
        if 'posts' not in self.objs:
            self.log.critical(_('``posts`` must be in parsed objects.'))
            return False
        return super(PostArchiveBase, self).write_preconditions()

    def path(self, **kwargs):
        '''Use the urlmapper to construct a path for the given attributes.
        '''
        urlfor = self.urlmapper.urlfor
        return urlfor(self.fmt, type='post', **kwargs)


class PostArchiveAll(PostArchiveBase):
    '''Parse the posts into a list, grouped by page.

    The :func:``render`` function should be overrridden to actually render the
    posts.

    This depends upon the objs having a value for the key ``posts``.

    '''

    fmt = 'html'

    def rev_key(self, key):
        '''Return a dictionary mapping attributes to values in the key.
        '''
        return {}

    def urls(self):
        '''A list of rooted paths that are the path component of URLs.

        Example on testdata/pristine::

        >>> urlmapper.add(components.Type('post')/components.pageno)
        >>> paa = PostArchiveAll(settings, objs, urlmapper)
        >>> pprint(paa.urls())
        ['index.html', 'page2/index.html', 'page3/index.html']

        '''
        ret = list()
        def action(post_list, pages):
            '''The action to pass to paginate_action.
            '''
            ret.append(self.path(page=pages.cur))
        paginate.paginate_action(self.settings.POSTS_PER_PAGE,
                self._sorted_posts(), action)
        return ret

    def write(self):
        '''Write the parsed posts to the filesystem.

        Example on testdata/pristine::

        >>> paa = PostArchiveAll(settings, objs, urlmapper)
        >>> paa.write()
        No previous page.
        Index 1
        Next 2
          - 2010-02-02-newday2
          - 2010-02-02-newday
        Prev 1
        Index 2
        Next 3
          - 2010-02-01-newmonth
          - 2010-01-01-newyear
        Prev 2
        Index 3
        No next page.
          - 2009-12-31-party

        '''
        paginate.paginate_action(self.settings.POSTS_PER_PAGE,
                self._sorted_posts(), self.render)

    def render(self, posts, pages):
        '''Render the view

        This should be overridden in child classes.
        '''
        if pages.prev is None:
            print 'No previous page.'
        else:
            print 'Prev', pages.prev
        print 'Index', pages.cur
        if pages.next is None:
            print 'No next page.'
        else:
            print 'Next', pages.next
        for post in posts:
            s = post.published.strftime('  - %Y-%m-%d-%%s')
            print s % post.slug


class PostArchiveYearly(PostArchiveBase):
    '''Parse the posts into a list, grouped by year, then by page.

    The :func:``render`` function should be overrridden to actually render the
    posts.

    This depends upon the objs having a value for the key ``posts``.

    '''

    fmt = 'html'

    def key(self, post):
        '''The key for the post.
        '''
        return (post.published.year,)

    def rev_key(self, key):
        '''Return a dictionary mapping attributes to values in the key.
        '''
        return {'year': key[0]}

    def urls(self):
        '''A list of rooted paths that are the path component of URLs.

        Example on testdata/pristine::

        >>> urlmapper.add(components.Type('post')/components.year/components.pageno)
        >>> pay = PostArchiveYearly(settings, objs, urlmapper)
        >>> pprint(pay.urls())
        ['2010/index.html', '2010/page2/index.html', '2009/index.html']

        '''
        ret = list()
        def action(obj_list, years, pages):
            '''The action to pass to split_paginate_action.
            '''
            ret.append(self.path(page=pages.cur, year=years.cur[0]))
        paginate.split_paginate_action(self.settings.POSTS_PER_PAGE,
                self.key, self._sorted_posts(), action)
        return ret

    def write(self):
        '''Write the parsed posts to the filesystem.

        Example on testdata/pristine::

        >>> pay = PostArchiveYearly(settings, objs, urlmapper)
        >>> pay.write()
        No previous year.
        Current year: 2010
        Next year: 2009
            No previous page.
            Index 1
            Next 2
             - 2010-02-02-newday2
             - 2010-02-02-newday
        No previous year.
        Current year: 2010
        Next year: 2009
            Prev 1
            Index 2
            No next page.
             - 2010-02-01-newmonth
             - 2010-01-01-newyear
        Previous year: 2010
        Current year: 2009
        No next year.
            No previous page.
            Index 1
            No next page.
             - 2009-12-31-party

        '''
        paginate.split_paginate_action(self.settings.POSTS_PER_PAGE,
                self.key, self._sorted_posts(), self.render)

    def render(self, posts, years, pages):
        '''Render the page corresponding to a single year/page.

        This should be overridden in child classes.
        '''
        if years.prev is None:
            print 'No previous year.'
        else:
            print 'Previous year: %s' % years.prev[0]
        print 'Current year: %s' % years.cur[0]
        if years.next is None:
            print 'No next year.'
        else:
            print 'Next year: %s' % years.next[0]

        if pages.prev is None:
            print '    No previous page.'
        else:
            print '    Prev', pages.prev
        print '    Index', pages.cur
        if pages.next is None:
            print '    No next page.'
        else:
            print '    Next', pages.next
        for post in posts:
            s = post.published.strftime('     - %Y-%m-%d-%%s')
            print s % post.slug


class PostArchiveMonthly(PostArchiveBase):
    '''Parse the posts into a list, grouped by month, then by page.

    The :func:``render`` function should be overrridden to actually render the
    posts.

    This depends upon the objs having a value for the key ``posts``.

    '''

    fmt = 'html'

    def key(self, post):
        '''The key for the post.
        '''
        return (post.published.year, post.published.month)

    def rev_key(self, key):
        '''Return a dictionary mapping attributes to values in the key.
        '''
        return {'year': key[0], 'month': key[1]}

    def urls(self):
        '''A list of rooted paths that are the path component of URLs.

        Example on testdata/pristine::

        >>> urlmapper.add(
        ...     components.Type('post')/components.year/components.month/components.pageno)
        >>> pam = PostArchiveMonthly(settings, objs, urlmapper)
        >>> pprint(pam.urls())
        ['2010/02/index.html',
         '2010/02/page2/index.html',
         '2010/01/index.html',
         '2009/12/index.html']

        '''
        ret = list()
        def action(obj_list, months, pages):
            '''The action to pass to split_paginate_action.
            '''
            ret.append(self.path(page=pages.cur, year=months.cur[0],
                month=months.cur[1]))
        paginate.split_paginate_action(self.settings.POSTS_PER_PAGE,
                self.key, self._sorted_posts(), action)
        return ret

    def write(self):
        '''Write the parsed posts to the filesystem.

        Example on testdata/pristine::

        >>> pam = PostArchiveMonthly(settings, objs, urlmapper)
        >>> pam.write()
        No previous month.
        Current month: 2010-2
        Next month: 2010-1
            No previous page.
            Index 1
            Next 2
             - 2010-02-02-newday2
             - 2010-02-02-newday
        No previous month.
        Current month: 2010-2
        Next month: 2010-1
            Prev 1
            Index 2
            No next page.
             - 2010-02-01-newmonth
        Previous month: 2010-2
        Current month: 2010-1
        Next month: 2009-12
            No previous page.
            Index 1
            No next page.
             - 2010-01-01-newyear
        Previous month: 2010-1
        Current month: 2009-12
        No next month.
            No previous page.
            Index 1
            No next page.
             - 2009-12-31-party

        '''
        paginate.split_paginate_action(self.settings.POSTS_PER_PAGE,
                self.key, self._sorted_posts(), self.render)

    def render(self, posts, months, pages):
        '''Render the page corresponding to a single month/page.

        This should be overridden in child classes.
        '''
        if months.prev is None:
            print 'No previous month.'
        else:
            print 'Previous month: %s-%s' % months.prev
        print 'Current month: %s-%s' % months.cur
        if months.next is None:
            print 'No next month.'
        else:
            print 'Next month: %s-%s' % months.next

        if pages.prev is None:
            print '    No previous page.'
        else:
            print '    Prev', pages.prev
        print '    Index', pages.cur
        if pages.next is None:
            print '    No next page.'
        else:
            print '    Next', pages.next
        for post in posts:
            fmt_str = post.published.strftime('     - %Y-%m-%d-%%s')
            print fmt_str % post.slug


class PostArchiveDaily(PostArchiveBase):
    '''Parse the posts into a list, grouped by day, then by page.

    The :func:``render`` function should be overrridden to actually render the
    posts.

    This depends upon the objs having a value for the key ``posts``.

    '''

    fmt = 'html'

    def key(self, post):
        '''The key for the post.
        '''
        return (post.published.year, post.published.month, post.published.day)

    def rev_key(self, key):
        '''Return a dictionary mapping attributes to values in the key.
        '''
        return {'year': key[0], 'month': key[1], 'day': key[2]}

    def urls(self):
        '''A list of rooted paths that are the path component of URLs.

        Example on testdata/pristine::

        >>> c = components
        >>> settings.POSTS_PER_PAGE = 1
        >>> urlmapper.add(
        ...     c.Type('post')/c.year/c.month/c.day/c.pageno)
        >>> pad = PostArchiveDaily(settings, objs, urlmapper)
        >>> pprint(pad.urls())
        ['2010/02/02/index.html',
         '2010/02/02/page2/index.html',
         '2010/02/01/index.html',
         '2010/01/01/index.html',
         '2009/12/31/index.html']

        '''
        ret = list()
        def action(obj_list, days, pages):
            '''The action to pass to split_paginate_action.
            '''
            ret.append(self.path(page=pages.cur, year=days.cur[0],
                month=days.cur[1], day=days.cur[2]))
        paginate.split_paginate_action(self.settings.POSTS_PER_PAGE,
                self.key, self._sorted_posts(), action)
        return ret

    def write(self):
        '''Write the parsed posts to the filesystem.

        Example on testdata/pristine::

        >>> settings.POSTS_PER_PAGE = 1
        >>> pad = PostArchiveDaily(settings, objs, urlmapper)
        >>> pad.write()
        No previous day.
        Current day: 2010-2-2
        Next day: 2010-2-1
            No previous page.
            Index 1
            Next 2
             - 2010-02-02-newday2
        No previous day.
        Current day: 2010-2-2
        Next day: 2010-2-1
            Prev 1
            Index 2
            No next page.
             - 2010-02-02-newday
        Previous day: 2010-2-2
        Current day: 2010-2-1
        Next day: 2010-1-1
            No previous page.
            Index 1
            No next page.
             - 2010-02-01-newmonth
        Previous day: 2010-2-1
        Current day: 2010-1-1
        Next day: 2009-12-31
            No previous page.
            Index 1
            No next page.
             - 2010-01-01-newyear
        Previous day: 2010-1-1
        Current day: 2009-12-31
        No next day.
            No previous page.
            Index 1
            No next page.
             - 2009-12-31-party

        '''
        paginate.split_paginate_action(self.settings.POSTS_PER_PAGE,
                self.key, self._sorted_posts(), self.render)

    def render(self, posts, days, pages):
        '''Render the page corresponding to a single day/page.

        This should be overridden in child classes.
        '''
        if days.prev is None:
            print 'No previous day.'
        else:
            print 'Previous day: %s-%s-%s' % days.prev
        print 'Current day: %s-%s-%s' % days.cur
        if days.next is None:
            print 'No next day.'
        else:
            print 'Next day: %s-%s-%s' % days.next

        if pages.prev is None:
            print '    No previous page.'
        else:
            print '    Prev', pages.prev
        print '    Index', pages.cur
        if pages.next is None:
            print '    No next page.'
        else:
            print '    Next', pages.next
        for post in posts:
            fmt_str = post.published.strftime('     - %Y-%m-%d-%%s')
            print fmt_str % post.slug


class PostSingle(PostWriter):
    '''Parse the posts into single pages.
    '''

    fmt = 'html'

    def path(self, post):
        '''Use the urlmapper to construct a path for the given attributes.
        '''
        urlfor = self.urlmapper.urlfor
        return urlfor(self.fmt, type='post', year=post.published.year,
                month=post.published.month, day=post.published.day,
                slug=post.slug)

    def url(self, post):
        '''Use the urlmapper to construct a URL for the given attributes.
        '''
        urlfor = self.urlmapper.urlfor
        return urlfor(self.fmt, absolute=True, type='post',
                year=post.published.year, month=post.published.month,
                day=post.published.day, slug=post.slug)

    def urls(self):
        '''A list of rooted paths that are the path component of URLs.

        Example on testdata/pristine::

            >>> c = components
            >>> urlmapper.add(c.Type('post')/c.year/c.month/c.day/c.slug)
            >>> ps = PostSingle(settings, objs, urlmapper)
            >>> pprint(ps.urls())
            ['2010/02/02/newday2/index.html',
             '2010/02/02/newday/index.html',
             '2010/02/01/newmonth/index.html',
             '2010/01/01/newyear/index.html',
             '2009/12/31/party/index.html']

        '''
        ret = list()
        for post in self._sorted_posts():
            ret.append(self.path(post=post))
        return ret

    def write(self):
        '''Write the parsed posts to the filesystem.

        Example on testdata/pristine::

        >>> ps = PostSingle(settings, objs, urlmapper)
        >>> ps.write()
        Post 2010/02/02/newday2
        Post 2010/02/02/newday
        Post 2010/02/01/newmonth
        Post 2010/01/01/newyear
        Post 2009/12/31/party

        '''
        for post in self._sorted_posts():
            self.render(post)

    def render(self, post):
        '''Render the posts.

        This should be overridden in child classes.
        '''
        print 'Post %s/%s' % (post.published.strftime('%Y/%m/%d'), post.slug)


def _setup(self):
    '''Setup the test cases.

    Actions taken::

        - Create a ``Settings`` object.
        - Create a ``Firmant`` object.
        - Load modules used in tests.

    '''
    from pysettings.settings import Settings
    from firmant.application import Firmant
    from firmant.routing import URLMapper
    from firmant.routing import components
    s = {'PARSERS': {'posts': 'firmant.parsers.posts.PostParser'}
        ,'CONTENT_ROOT': 'testdata/pristine'
        ,'POSTS_SUBDIR': 'posts'
        ,'REST_EXTENSION': 'rst'
        ,'POSTS_PER_PAGE': 2
        ,'PERMALINK_ROOT': 'http://test'
        }
    settings               = Settings(s)
    firmant                = Firmant(settings)
    firmant.parse()
    self.globs['settings'] = settings
    self.globs['objs']  = firmant.objs
    self.globs['urlmapper'] = URLMapper(root=settings.PERMALINK_ROOT)
    self.globs['components'] = components



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import logging
import stat
import tempfile
import os

from docutils import io
from docutils.core import publish_programmatically

from firmant.du import MetaDataStandaloneReader
from firmant.i18n import _
from firmant.utils import class_name
from firmant.utils.exceptions import log_uncaught_exceptions


__all__ = ['Parser']


class Parser(object):
    '''Interpret directory structures on the filesystem.

    A parser will convert a directory hierarchy on the file system into an
    internal representation that will be available to general transformation
    functions and writers.

    Example parsers include those that read posts, and those that find images.

    This class is meant to be used as a base, with the child implementing
    ``paths``, and ``parse_one``.

    The ``parse`` method is careful to trap tracebacks, and uses
    firmant.utils.exceptions.log_uncaught_exceptions to do so.

    '''

    def __init__(self, settings=None):
        '''Instantiate a Parser, bound to the settings.
        '''
        self.settings = settings
        self.log = logging.getLogger(class_name(self.__class__))

    def parse(self):
        '''Create a list of parsed objects.
        '''
        ret = list()
        for path in self.paths():
            def act():
                obj = self.parse_one(path)
                ret.append(obj)
            error = _('error parsing %s') % path
            save  = getattr(self.settings, 'SAVE_TRACEBACK', False)
            if log_uncaught_exceptions(act, self.log, error, save):
                self.log.info(_('parsed %s') % path)
        return ret

    def paths(self):
        '''Return a list of paths to objects on the file system.
        '''
        return list()

    def parse_one(self, path):
        '''Transform one path on the filesystem into a parsed object.
        '''
        return path


class RstObject(object):
    '''An object representing a parsed restructured text document.
    '''


class RstParser(Parser):
    '''Interpret *.rst for a given directory.

    For each reSt document, parse it to a doctree and extract its metadata.
    '''

    auto_metadata = []
    '''Attributes which should be automatically read from metadata.

    This value should be a list of 2-tuples where the first value is the
    attribute on the entry, and the second is the key to the metadata
    dictionary.
    '''

    auto_pubparts = []
    '''Attributes which should be automatically read from publisher parts.

    This value should be a list of 2-tuples where the first value is the
    attribute on the entry, and the second is the key to the publisher's parts
    dictionary.
    '''

    def new_object(self, path, d, pub):
        '''Return an instance of the object to which rst documents are parsed.
        '''
        return RstObject()

    def default(self, attr):
        '''Return the default value of an attribute.

        This base class version is limited::

            >>> r = RstParser()
            >>> r.default('foo') is None
            True

        It is intended that this function will be overridden (possibly to read
        from settings).

        '''
        return None

    def post_process(self, doc):
        '''Post-process the document.
        '''

    def parse_one(self, path):
        '''Transform one path on the filesystem into a parsed object.
        '''
        d = dict()
        args = {'source': None
               ,'source_path': path
               ,'source_class': io.FileInput
               ,'destination_class': io.StringOutput
               ,'destination': None
               ,'destination_path': None
               ,'reader': MetaDataStandaloneReader(data=d), 'reader_name': None
               ,'parser': None, 'parser_name': 'restructuredtext'
               ,'writer': None, 'writer_name': 'html'
               ,'settings': None, 'settings_spec': None
               ,'settings_overrides': None
               ,'config_section': None
               ,'enable_exit_status': None
               }
        null, pub = publish_programmatically(**args)

        # Create a new object, using information from path and dictionary to
        # fill in all values not filled by auto_metadata or the defaults.
        o = self.new_object(path, d, pub)

        # Automatically assign all auto_metadata to new object.
        for attr, directive in self.auto_metadata:
            default = self.default(attr)
            val = d.get(directive, None) or default
            setattr(o, attr, val)

        # Automatically assign all auto_pubparts to new object.
        for attr, directive in self.auto_pubparts:
            default = self.default(attr)
            val = pub.writer.parts.get(directive, None) or default
            setattr(o, attr, val)

        # Save the doctree for future use.
        o.document = pub.document

        # Post process the document.
        self.post_process(o)

        return o



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''This module handles mapping between paths and writers.

Writers are able to request a URL for a given set of attributes.
'''


import os.path

from firmant.utils import class_name
from firmant.utils import merge_dicts


class AbstractPath(object):
    '''The path component of a URL.

    This may be a single component, several components, or a whole URL.

    Intentionally the following methods and attributes raise errors indicating
    that they have not been implemented::

        >>> ap = AbstractPath()
        >>> ap.attributes
        Traceback (most recent call last):
        RuntimeError: Not Implemented
        >>> ap.bound_attributes
        Traceback (most recent call last):
        RuntimeError: Not Implemented
        >>> ap.construct()
        Traceback (most recent call last):
        RuntimeError: Not Implemented

    '''

    @property
    def attributes(self):
        raise RuntimeError("Not Implemented")

    @property
    def bound_attributes(self):
        raise RuntimeError("Not Implemented")

    @property
    def free_attributes(self):
        return self.attributes - set(self.bound_attributes.keys())

    def match(self, *args, **kwargs):
        # Provided values must exactly cover all attributes and not conflict
        # with bound attributes.
        d = merge_dicts(kwargs, *args)

        # Check for conflict with bound values
        try:
            merge_dicts(d, self.bound_attributes)
        except ValueError:
            return False

        a = set(self.attributes)
        b = set(d.keys())
        return a & b == a | b

    def construct(self, *args, **kwargs):
        raise RuntimeError("Not Implemented")

    def __div__(self, rhs):
        return CompoundComponent(self, rhs)


class SinglePathComponent(AbstractPath):
    '''A single piece of a URL.

    Example usage::

        >>> spc = SinglePathComponent('month', lambda m: '%02i' % m)
        >>> spc.attributes
        set(['month'])
        >>> spc.bound_attributes
        {}
        >>> spc.free_attributes
        set(['month'])
        >>> spc.match({'month': 3})
        True
        >>> spc.match(month=3)
        True
        >>> spc.match(year=2010)
        False
        >>> spc.match(year=2010, month=3)
        False
        >>> spc.construct({'month': 3})
        '03'
        >>> spc.construct(month=3)
        '03'

    Error conditions::

        >>> spc.match({'month': 2}, month=3)
        Traceback (most recent call last):
        ValueError: Conflicting values for 'month'
        >>> spc.construct(month=3, year=10)
        Traceback (most recent call last):
        ValueError: Attributes do not match URL

    '''

    def __init__(self, attribute, conv=lambda x:x):
        self._attribute = attribute
        self._conv = conv

    @property
    def attributes(self):
        return set([self._attribute])

    @property
    def bound_attributes(self):
        return dict()

    def construct(self, *args, **kwargs):
        if not self.match(*args, **kwargs):
            raise ValueError('Attributes do not match URL')
        d = merge_dicts(kwargs, *args)
        return self._conv(d[self._attribute])


class BoundNullPathComponent(AbstractPath):
    '''A path component that does not impact the URL.

    In this example, a path component is created that will match if the
    attribute `month` is 3::

        >>> bnpc = BoundNullPathComponent('month', 3)
        >>> bnpc.attributes
        set(['month'])
        >>> bnpc.bound_attributes
        {'month': 3}
        >>> bnpc.free_attributes
        set([])
        >>> bnpc.match({'month': 3})
        True
        >>> bnpc.match(month=3)
        True
        >>> bnpc.construct({'month': 3}) is None
        True
        >>> bnpc.construct(month=3) is None
        True

    '''

    def __init__(self, attribute, value):
        self._attribute = attribute
        self._value = value

    @property
    def attributes(self):
        return set([self._attribute])

    @property
    def bound_attributes(self):
        d = dict()
        d[self._attribute] = self._value
        return d

    def construct(self, *args, **kwargs):
        if not self.match(*args, **kwargs):
            raise ValueError('Attributes do not match URL')
        return None


class StaticPathComponent(AbstractPath):
    '''A path component that does not have any attributes.

    In this example, a path component is created that will construct to the path
    ``images``::

        >>> spc = StaticPathComponent('images')
        >>> spc.attributes
        set([])
        >>> spc.bound_attributes
        {}
        >>> spc.free_attributes
        set([])
        >>> spc.match({'month': 3})
        False
        >>> spc.match()
        True
        >>> spc.construct()
        'images'

    '''

    def __init__(self, path):
        self._path = str(path)

    @property
    def attributes(self):
        return set([])

    @property
    def bound_attributes(self):
        return {}

    def construct(self, *args, **kwargs):
        if not self.match(*args, **kwargs):
            raise ValueError('Attributes do not match URL')
        return self._path


class CompoundComponent(AbstractPath):
    '''A compound path formed using zero or more AbstractPath objects.

    Example usage::

        >>> year = SinglePathComponent('year', lambda y: '%04i' % y)
        >>> month = SinglePathComponent('month', lambda m: '%02i' % m)
        >>> day = SinglePathComponent('day', lambda d: '%02i' % d)
        >>> ymd = year/month/day
        >>> ymd #doctest: +ELLIPSIS
        <firmant.routing.CompoundComponent object at 0x...>
        >>> ymd.match(year=2010, month=3, day=14)
        True
        >>> ymd.match(year=2010, month=3, day=14, slug='foobar')
        False
        >>> ymd.construct(year=2010, month=3, day=14)
        '2010/03/14'

    With a null path component::

        >>> bnpc = BoundNullPathComponent('type', 'tag')
        >>> slug = SinglePathComponent('slug', str)
        >>> path = bnpc/slug
        >>> path #doctest: +ELLIPSIS
        <firmant.routing.CompoundComponent object at 0x...>
        >>> path.attributes
        set(['type', 'slug'])
        >>> path.bound_attributes
        {'type': 'tag'}
        >>> path.free_attributes
        set(['slug'])
        >>> path.match(slug='foobar')
        False
        >>> path.match(type='tag', slug='foobar')
        True
        >>> path.construct(type='tag', slug='foobar')
        'foobar'

    With a static path component::

        >>> spc = StaticPathComponent('images')
        >>> slug = SinglePathComponent('slug', str)
        >>> path = spc/slug
        >>> path #doctest: +ELLIPSIS
        <firmant.routing.CompoundComponent object at 0x...>
        >>> path.attributes
        set(['slug'])
        >>> path.bound_attributes
        {}
        >>> path.free_attributes
        set(['slug'])
        >>> path.match(slug='foobar')
        True
        >>> path.match(type='tag', slug='foobar')
        False
        >>> path.construct(slug='foobar')
        'images/foobar'

    '''

    def __init__(self, *args):
        self._components = args

    @property
    def attributes(self):
        list_of_sets = map(lambda x: x.attributes, self._components)
        return reduce(set.__or__, list_of_sets)

    @property
    def bound_attributes(self):
        list_of_sets = map(lambda x: x.bound_attributes, self._components)
        return reduce(merge_dicts, list_of_sets)

    def construct(self, *args, **kwargs):
        if not self.match(*args, **kwargs):
            raise ValueError('Attributes do not match URL')
        d = merge_dicts(kwargs, *args)
        list_of_paths = map(lambda x: self._call_construct(x, d),
                self._components)
        list_of_paths = filter(bool, list_of_paths)
        return '/'.join(list_of_paths)

    @classmethod
    def _call_construct(cls, component, d):
        e = d.copy()
        for key in e.keys():
            if key not in component.attributes:
                e.pop(key, None)
        return component.construct(e)


class URLMapper(object):
    '''Convert between attributes and paths.

    Example::

        >>> # setup (this would be done by the Firmant object)
        >>> post = BoundNullPathComponent('type', 'post')
        >>> year = SinglePathComponent('year', lambda y: '%04i' % y)
        >>> month = SinglePathComponent('month', lambda m: '%02i' % m)
        >>> day = SinglePathComponent('day', lambda d: '%02i' % d)
        >>> slug = SinglePathComponent('slug', str)
        >>> from pysettings.settings import Settings
        >>> um = URLMapper(root='http://test')
        >>> um.add( post/year/month/day/slug )
        >>> um.add( post/year/month/day )
        >>> um.add( post/year/month )
        >>> um.add( post/year )

        >>> # usage (this would be done within writers/transformers)
        >>> um.lookup(type='post', slug='foobar', day=15, month=3, year=2010)
        '2010/03/15/foobar'
        >>> um.lookup(type='post', year=2010)
        '2010'
        >>> um.lookup(type='post', unknown_attribute=True)
        Traceback (most recent call last):
        AttributeError: Attributes do not correspond to any path
        >>> um.urlfor('html', type='post', slug='foobar', day=15, month=3, year=2010)
        '2010/03/15/foobar/index.html'
        >>> um.urlfor('html', type='post', year=2010)
        '2010/index.html'
        >>> um.urlfor('html', type='post', unknown_attribute=True) is None
        True

    If `absolute` is True, the URL will be prefixed with the root (if it was
    specified).

        >>> um.urlfor('html', type='post', absolute=True, slug='foobar', day=15, month=3, year=2010)
        'http://test/2010/03/15/foobar/index.html'

    If `format` is '', then the path formed by the attributes is kept as is and is
    not interpreted as a directory with an 'index.%s' % format path appended::

        >>> um.urlfor('', type='post', absolute=True, slug='foobar', day=15, month=3, year=2010)
        'http://test/2010/03/15/foobar'

    '''

    def __init__(self, urls=None, root=None):
        self.root=root
        self._paths = list()
        if urls is not None:
            self._paths.extend(urls)

    def add(self, path):
        self._paths.append(path)

    def lookup(self, **kwargs):
        '''Lookup the abstract representation of a page defined by attributes.

        Unless you know to do otherwise, use :method:`urlfor` instead of this
        method.
        '''
        for path in self._paths:
            if path.match(kwargs):
                return path.construct(kwargs)
        raise AttributeError('Attributes do not correspond to any path')

    def urlfor(self, format, absolute=False, **kwargs):
        '''Return the path of a page relative to the content root.

        Unless you know to do otherwise, use this method instead of
        :method:`lookup`.

        '''
        try:
            path = self.lookup(**kwargs)
        except AttributeError:
            return None
        path = path or ''
        if absolute and self.root is None:
            raise RuntimeError('Set the root for absolute URLs')
        if format != '':
            path = os.path.join(path, 'index.%s' % format)
        if absolute:
            return self.absolute(path)
        else:
            return path

    def absolute(self, path):
        '''Return the absolute url for the path.
        '''
        return os.path.join(self.root, path)



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''Writers create the filesystem to be published.

Each writer will handle a subset of the objs, and create a portion of the
outputted directory hierarchy.
'''


import abc
import collections
import logging

from firmant.i18n import _
from firmant.utils import class_name


class Writer(object):
    '''Handle writing parsed objects to the filesystem.

    It is assumed that a writer will have the following methods::

        - write:  Write all objects to the filesystem.
        - write_preconditions:  Tests that the writer is able to write.  If this
          returns False, it is likely that the writer would fail.
    '''

    __metaclass__ = abc.ABCMeta

    def __init__(self, settings, objs, urlmapper):
        self.settings = settings
        self.objs = objs
        self.urlmapper = urlmapper
        self.log = logging.getLogger(class_name(self.__class__))

    @abc.abstractmethod
    def urls(self):
        '''A list of paths that the writer will write.

        Each path is assumed to be relative to the webroot of the blog.  It is
        implicit that paths begin with '/' (e.g. the path 'foo/bar' translates
        implies a request URI of '/foo/bar').
        '''

    @abc.abstractmethod
    def write(self):
        '''Write the objects to the filesystem.
        '''

    def write_preconditions(self):
        '''Returns true if and only if it is acceptable to proceed with writing.
        '''
        # Fail if we do not have an output directory.
        if getattr(self.settings, 'OUTPUT_DIR', None) is None:
            self.log.critical(_('``OUTPUT_DIR`` not defined in settings.'))
            return False
        return True



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''Writers that write various views of the feeds.
'''


from copy import copy

from firmant.writers import Writer


class FeedWriter(Writer):
    '''Base class used for functionality common to all feed writers.
    '''


class FeedSingle(FeedWriter):
    '''Parse the posts into single feed documents.
    '''

    fmt = 'atom'

    def path(self, feed):
        '''Use the urlmapper to construct a path for the given attributes.
        '''
        urlfor = self.urlmapper.urlfor
        return urlfor(self.fmt, type='feed', slug=feed.slug)

    def url(self, feed):
        '''Use the urlmapper to construct a URL for the given attributes.
        '''
        urlfor = self.urlmapper.urlfor
        return urlfor(self.fmt, absolute=True, type='feed', slug=feed.slug)

    def urls(self):
        '''A list of rooted paths that are the path component of URLs.

        Example on testdata/pristine::

            >>> c = components
            >>> urlmapper.add(c.Type('feed')/c.slug)
            >>> fs = FeedSingle(settings, objs, urlmapper)
            >>> pprint(fs.urls())
            ['bar/index.atom', 'baz/index.atom', 'foo/index.atom', 'quux/index.atom']

        '''
        ret = list()
        for feed in self.objs.get('feeds', []):
            ret.append(self.path(feed=feed))
        ret.sort()
        return ret

    def write(self):
        '''Write a parsed feed to the filesystem.

        Example on testdata/pristine::

            >>> fs = FeedSingle(settings, objs, urlmapper)
            >>> fs.write()
            Feed bar
              2010/01/01/newyear
              2009/12/31/party
            Feed baz
              2010/02/02/newday2
              2010/02/02/newday
              2010/02/01/newmonth
            Feed foo
              2010/02/02/newday2
              2010/02/02/newday
              2010/01/01/newyear
            Feed quux
              2010/02/01/newmonth
              2009/12/31/party

        '''
        for feed in self.objs.get('feeds', []):
            feed  = copy(feed)
            posts = [x for x in reversed(sorted(feed.posts,
                    key=lambda p: (p.updated, p.published.date(), p.slug)))]
            feed.posts = posts[:self.settings.POSTS_PER_FEED]
            self.render(feed)

    def render(self, feed):
        '''Render the feed.

        This should be overridden in child classes.
        '''
        print 'Feed %s' % feed.slug
        for post in feed.posts:
            print '  %s/%s' % (post.published.strftime('%Y/%m/%d'), post.slug)


def _setup(self):
    '''Setup the test cases.

    Actions taken::

        - Create a ``Settings`` object.
        - Create a ``Firmant`` object.
        - Load modules used in tests.

    '''
    from pysettings.settings import Settings
    from firmant.application import Firmant
    from firmant.routing import URLMapper
    from firmant.routing import components
    s = {'PARSERS': {'posts': 'firmant.parsers.posts.PostParser'
                    ,'feeds': 'firmant.parsers.feeds.FeedParser'
                    ,'tags': 'firmant.parsers.tags.TagParser'
                    }
        ,'CONTENT_ROOT': 'testdata/pristine'
        ,'POSTS_SUBDIR': 'posts'
        ,'FEEDS_SUBDIR': 'feeds'
        ,'TAGS_SUBDIR': 'feeds'
        ,'REST_EXTENSION': 'rst'
        ,'POSTS_PER_PAGE': 2
        ,'POSTS_PER_FEED': 3
        }
    settings               = Settings(s)
    firmant                = Firmant(settings)
    from minimock import Mock
    firmant.parse()
    firmant.cross_reference()
    self.globs['settings'] = settings
    self.globs['objs']  = firmant.objs
    self.globs['urlmapper'] = URLMapper()
    self.globs['components'] = components



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''Render feed in the `Atom format
<http://www.atomenabled.org/developers/syndication/atom-format-spec.php>`_.

'''


import datetime
import os

from lxml import etree

from firmant.utils import paths
from firmant.writers.feeds import FeedSingle


def add_text_subelement(root, name, text):
    new = etree.SubElement(root, name)
    new.text = text


def RFC3339(dt):
    frmt_str = '%Y-%m-%dT%H:%M:%S'
    timestamp = dt.strftime(frmt_str)
    timezone  = dt.strftime('%z')
    if timezone != '':
        return timestamp + timezone[:-2] + ':' + timezone[-2:]
    else:
        return timestamp + 'Z'


class AtomFeedSingle(FeedSingle):

    permalinks_for = 'feeds'

    def render(self, feed_obj):
        '''Render the feed according to the Atom specification.
        '''
        feed = etree.Element('feed')
        feed.set('xmlns', 'http://www.w3.org/2005/Atom')

        add_text_subelement(feed, 'generator', 'Firmant')
        add_text_subelement(feed, 'title', feed_obj.title)
        add_text_subelement(feed, 'rights', feed_obj.copyright)
        updated = max([post.published for post in feed_obj.posts] +
                [datetime.datetime(1900, 01, 01)])
        add_text_subelement(feed, 'updated', RFC3339(updated))
        add_text_subelement(feed, 'id', feed_obj.permalink)
        link = etree.SubElement(feed, 'link')
        link.set('href', feed_obj.permalink)
        link.set('rel', 'self')

        for post_obj in feed_obj.posts:
            post = etree.SubElement(feed, 'entry')
            add_text_subelement(post, 'title', post_obj.title)
            add_text_subelement(post, 'updated', RFC3339(post_obj.updated))
            add_text_subelement(post, 'published', RFC3339(post_obj.published))

            author = etree.SubElement(post, 'author')
            add_text_subelement(author, 'name', post_obj.author)

            content      = etree.SubElement(post, 'content')
            content.text = post_obj.content
            content.set('type', 'html')

            l_alt = etree.SubElement(post, 'link')
            l_alt.set('href', post_obj.permalink)
            l_alt.set('rel', 'alternate')

            add_text_subelement(post, 'id', post_obj.permalink)
            add_text_subelement(post, 'rights', post_obj.copyright)

            for tag in post_obj.tags:
                category = etree.SubElement(post, 'category')
                category.set('term', tag.slug)
                category.set('label', tag.title)

        url = self.path(feed_obj)
        data = etree.tostring(feed)
        path = os.path.join(self.settings.OUTPUT_DIR, url or '')
        f    = paths.create_or_truncate(path)
        f.write(data.encode('utf-8'))
        f.flush()
        f.close()



from distutils.core import setup

setup(name='Firmant',
        version='0.1.2',
        description='Static Blog Generator',
        author='Robert Escriva',
        author_email='projects@mail.robescriva.com',
        url='http://projects.robescriva.com/projects/show/firmant',
        packages=['firmant', 'firmant.parsers', 'firmant.routing',
            'firmant.utils', 'firmant.writers'],
        )



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''Utilities for manipulating paths/filesystems.
'''


import os
import sys


def cat(path, out=sys.stdout):
    r'''Write the contents of file ``path`` to ``out``.

        >>> m = Mock('output')
        >>> cat('testdata/settings/empty.py', m) #doctest: +ELLIPSIS
        Called output.write('# Copyright (c) 2010, Robert Escriva\n')
        ...
        Called output.write(
            '# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.\n')

    '''
    f = open(path)
    for line in f:
        print >> out, line,
    f.close()


def create_or_truncate(path):
    r'''Return an ``path`` as an open file, creating dirs if necessary.

    Example::

        >>> import tempfile
        >>> root = tempfile.mkdtemp()
        >>> path = os.path.join(root, 'foo/bar/baz/quux')
        >>> f = create_or_truncate(path)
        >>> f.write('THIS IS THE FIRST TIME\n')
        >>> f.flush() and f.close()

        >>> m = Mock('output')
        >>> cat(path, m)
        Called output.write('THIS IS THE FIRST TIME\n')

        >>> f2 = create_or_truncate(path)
        >>> f2.write('THIS IS THE SECOND TIME\n')
        >>> f2.flush() and f2.close()
        >>> cat(path, m)
        Called output.write('THIS IS THE SECOND TIME\n')

        >>> # Cleanup after ourselves.
        >>> import shutil
        >>> shutil.rmtree(root)

    '''
    par = os.path.dirname(path)
    if par != '':
        try:
            os.makedirs(par)
        except OSError, e:
            if e.errno != 17:
                raise
    return open(path, 'w+')



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''Paginate or group objects according to certain properties.
'''


class Paginated(object):
    '''An object representing keys in paginated lists.

    The key used for each of the lists is stored in the properties ``prev``,
    ``cur``, ``next``.  Instances are just nice containers in place of
    3-tuples::

        >>> p = Paginated(1, 2, 3)
        >>> p.prev
        1
        >>> p.cur
        2
        >>> p.next
        3

    '''

    def __init__(self, prev, cur, nex):
        self._prev = prev
        self._cur = cur
        self._next = nex

    @property
    def prev(self):
        '''The key of the previous list.
        '''
        return self._prev

    @property
    def cur(self):
        '''The key of the current list.
        '''
        return self._cur

    @property
    def next(self):
        '''The key of the next list.
        '''
        return self._next


def split_list_action(key_func, obj_list, action):
    '''Split obj_list at boundaries determined by key_func and call action.

    At each point the value of key_func changes, split obj_list.  Then for each
    new list, call action with the list, and the key values for prev/cur/next.

    The function's behavior is undefined if key_func ever returns None.

        >>> def parity(x):
        ...     return x % 2
        >>> def action(obj_list, groups):
        ...     print obj_list, groups.prev, groups.cur, groups.next
        >>> split_list_action(parity, [1, 3, 5, 2, 4, 7, 8, 9, 10], action)
        [1, 3, 5] None 1 0
        [2, 4] 1 0 1
        [7] 0 1 0
        [8] 1 0 1
        [9] 0 1 0
        [10] 1 0 None
        >>> split_list_action(parity, [], action)
        >>> split_list_action(parity, [1], action)
        [1] None 1 None

    '''
    cur_list = obj_list
    nex_list = cur_list[1:] + [None]

    kprev = None

    acc = list()
    for cur, nex in zip(cur_list, nex_list):
        kcur = cur and key_func(cur)
        knex = nex and key_func(nex)
        acc.append(cur)
        if kcur != knex:
            action(acc, Paginated(kprev, kcur, knex))
            kprev = kcur
            acc = list()


def paginate_action(num_per_page, obj_list, action):
    '''Call ``action`` with the prev/cur/next page numbers and an obj_list.

        >>> def action(obj_list, pages):
        ...     print obj_list, pages.prev, pages.cur, pages.next
        >>> paginate_action(1, [1, 2, 3, 4, 5, 6, 7], action)
        [1] None 1 2
        [2] 1 2 3
        [3] 2 3 4
        [4] 3 4 5
        [5] 4 5 6
        [6] 5 6 7
        [7] 6 7 None
        >>> paginate_action(2, [1, 2, 3, 4, 5, 6, 7], action)
        [1, 2] None 1 2
        [3, 4] 1 2 3
        [5, 6] 2 3 4
        [7] 3 4 None
        >>> paginate_action(2, [], action)
        >>> paginate_action(2, [1], action)
        [1] None 1 None

    '''
    num_pages = (len(obj_list) + num_per_page - 1) / num_per_page

    prev_list = [None] + range(1, num_pages)
    cur_list  = range(1, num_pages + 1)
    nex_list  = range(2, num_pages + 1) + [None]

    for prev, cur, nex in zip(prev_list, cur_list, nex_list):
        begin = (cur - 1) * num_per_page
        end   = begin + num_per_page
        objs  = obj_list[begin:end]
        action(objs, Paginated(prev, cur, nex))


def split_paginate_action(num_per_page, key_func, obj_list, action):
    '''Split obj_list with split_list_action; then use paginate_action.

    The lists will be split according to split_list_action.  Each of the
    resulting lists will then be passed to paginate_action.  The result will be
    the action callable will be called with arguments for the objects, the
    prev/cur/next keys from split_list_action, and the prev/cur/next keys from
    paginate_action (total of 7 args).

        >>> def parity(x):
        ...     return x % 2
        >>> def action(obj_list, groups, pages):
        ...     print obj_list, groups.prev, groups.cur, groups.next,
        ...     print pages.prev, pages.cur, pages.next
        >>> split_paginate_action(2, parity, [1, 3, 5, 7, 9, 2, 4, 7, 9, 10], action)
        [1, 3] None 1 0 None 1 2
        [5, 7] None 1 0 1 2 3
        [9] None 1 0 2 3 None
        [2, 4] 1 0 1 None 1 None
        [7, 9] 0 1 0 None 1 None
        [10] 1 0 None None 1 None
        >>> split_paginate_action(2, parity, [], action)
        >>> split_paginate_action(2, parity, [1], action)
        [1] None 1 None None 1 None
        >>> split_paginate_action(2, parity, [1, 2], action)
        [1] None 1 0 None 1 None
        [2] 1 0 None None 1 None

    '''
    def new_act_split_list(obj_list, groups):
        '''The action to pass to split_list_action.
        '''
        def new_act_paginate(obj_list, pages):
            '''The action to pass to paginate_action.
            '''
            action(obj_list, groups, pages)
        paginate_action(num_per_page, obj_list, new_act_paginate)
    split_list_action(key_func, obj_list, new_act_split_list)



# Copyright (c) 2010, Robert Escriva
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Firmant nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''Manipulate post objects for Firmant.
'''


import datetime
import os

from firmant.parsers import RstParser
from firmant.parsers import RstObject
from firmant.utils import strptime


__all__ = ['PostParser']


class PostParser(RstParser):
    '''Interpret *.rst for a given directory.

    For each reSt document, parse it to a post and extract its metadata.

        >>> p = PostParser()
        >>> p = p.parse_one('content/posts/1975-03-23-give-me-liberty.rst')
        >>> p.slug
        u'give-me-liberty'
        >>> p.published
        datetime.datetime(1975, 3, 23, 13, 1)
        >>> p.author
        u'Patrick Henry'
        >>> p.tags
        [u'speech', u'patriotism']
        >>> p.feeds
        [u'patriotic-things']
        >>> p.copyright
        u'This document is part of the public domain.'
        >>> p.updated
        datetime.datetime(2009, 2, 17, 11, 31)
        >>> p.title
        u'Give Me Liberty or Give Me Death'
        >>> p.content #doctest: +ELLIPSIS
        u"<p>Two hundred years ago people ... or give me death.</p>\\n"
        >>> p.tz
        u'US/Eastern'

    Default values (except published/updated/slug)::

        >>> p = PostParser()
        >>> p = p.parse_one('content/posts/2010-02-15-empty.rst')
        >>> p.slug
        u'empty'
        >>> p.published
        datetime.datetime(2010, 2, 15, 0, 0)
        >>> p.author
        u''
        >>> p.tags
        []
        >>> p.feeds
        []
        >>> p.copyright
        u''
        >>> p.updated
        datetime.datetime(2010, 2, 15, 0, 0)
        >>> p.title
        u''
        >>> p.content #doctest: +ELLIPSIS
        u''
        >>> p.tz
        u''

    All posts may be retrieved with::

        >>> from pysettings.settings import Settings
        >>> s = {'CONTENT_ROOT': 'content'
        ...     ,'POSTS_SUBDIR': 'posts'
        ...     ,'REST_EXTENSION': 'rst'
        ...     }
        >>> s = Settings(s)
        >>> p = PostParser(s)
        >>> p.log = Mock('log')
        >>> pprint(map(lambda p: (p.published.date(), p.slug), p.parse()))
        Called log.info('parsed content/posts/1975-03-23-give-me-liberty.rst')
        Called log.info('parsed content/posts/2009-02-17-loren-ipsum.rst')
        Called log.info('parsed content/posts/2010-02-13-sample-code.rst')
        Called log.info('parsed content/posts/2010-02-15-empty.rst')
        [(datetime.date(1975, 3, 23), u'give-me-liberty'),
         (datetime.date(2009, 2, 17), u'loren-ipsum'),
         (datetime.date(2010, 2, 13), u'sample-code'),
         (datetime.date(2010, 2, 15), u'empty')]

    '''

    auto_metadata = [('author', 'author')
                    ,('tags', 'tags')
                    ,('feeds', 'feeds')
                    ,('copyright', 'copyright')
                    ,('updated', 'updated')
                    ,('tz', 'timezone')
                    ]

    auto_pubparts = [('title', 'title')
                    ,('content', 'fragment')
                    ]

    def paths(self):
        '''Return a list of paths to objects on the file system.

        Consider all the contents of the posts directory (by default this is
        ``content_root/posts``).  Only files ending in ``suffix`` are
        considered.

        Directory entries that are not files are ignored.

            >>> from pysettings.settings import Settings
            >>> s = {'CONTENT_ROOT': 'content'
            ...     ,'POSTS_SUBDIR': 'posts'
            ...     ,'REST_EXTENSION': 'rst'
            ...     }
            >>> s = Settings(s)
            >>> p = PostParser(s)
            >>> pprint(p.paths())
            ['content/posts/1975-03-23-give-me-liberty.rst',
             'content/posts/2009-02-17-loren-ipsum.rst',
             'content/posts/2010-02-13-sample-code.rst',
             'content/posts/2010-02-15-empty.rst']

        '''
        # TODO add above settings to global settings.
        settings = self.settings
        path  = os.path.join(settings.CONTENT_ROOT, settings.POSTS_SUBDIR)
        files = os.listdir(path)
        files = map(lambda p: os.path.join(path, p), files)
        files = filter(lambda f: f.endswith(settings.REST_EXTENSION), files)
        files = filter(lambda f: os.path.isfile(f), files)
        files.sort()
        return files

    def new_object(self, path, d, pub):
        '''Return an instance of the object to which rst documents are parsed.
        '''
        p = RstObject()
        p.slug, null = os.path.basename(path)[11:].rsplit('.', 1)
        p.slug = unicode(p.slug)
        dt = strptime(os.path.basename(path)[:10], ['%Y-%m-%d'])
        if 'time' in d:
            t = d['time']
        else:
            t = datetime.time(0, 0, 0)
        p.published = datetime.datetime.combine(dt.date(), t)
        p.updated = p.published
        return p

    def default(self, attr):
        '''Return the default value of an attribute.
        '''
        d = {'tags': list()
            ,'feeds': list()
            }
        return d.get(attr, u'')

    def post_process(self, doc):
        '''Cleanup the update timestamp.
        '''
        if doc.updated == u'':
            doc.updated = doc.published
